@@ -5,6 +5,7 @@ from typing_extensions import (
     Protocol,
     runtime_checkable,
     TypedDict,
+    Generic,
 )
 from typing import (
     List,
@@ -21,9 +22,8 @@ from typing import (
     Awaitable,
 )
 
-from rx.core.typing import Observable
+from reactivex import Observable
 from PIL import Image
-import numpy as np
 
 
 Callback = Union[
@@ -282,11 +282,41 @@ class Camera(BaseCamera):
     sources: List[CameraInput]
 
 
+CameraPublicProperties = Literal[
+    "name",
+    "disabled",
+    "isCloud",
+    "hasLight",
+    "hasSiren",
+    "hasBinarySensor",
+    "hasBattery",
+    "info",
+    "type",
+    "motionZones",
+    "objectZones",
+    "hasAudioDetector",
+    "hasMotionDetector",
+    "hasObjectDetector",
+    "hasPrebuffer",
+    "hasIntercom",
+    "hasPtz",
+    "sources",
+]
+
+PropertyValue = Union[
+    str,
+    bool,
+    CameraInformation,
+    CameraType,
+    List[CameraZone],
+    List[CameraInput],
+]
+
+
 # camera device
 class BaseDetection(TypedDict):
     id: NotRequired[str]
     label: str
-    timestamp: int
     confidence: float
     boundingBox: Tuple[int, int, int, int]
     inputWidth: int
@@ -302,42 +332,42 @@ class ObjectDetection(BaseDetection): ...
 
 
 class BaseStateLight(TypedDict):
-    timestamp: int
+    timestamp: str
     lastEvent: NotRequired[Optional["LightStateWithoutLastEvent"]]
 
 
 class BaseStateMotion(TypedDict):
-    timestamp: int
+    timestamp: str
     lastEvent: NotRequired[Optional["MotionStateWithoutLastEvent"]]
 
 
 class BaseStateAudio(TypedDict):
-    timestamp: int
+    timestamp: str
     lastEvent: NotRequired[Optional["AudioStateWithoutLastEvent"]]
 
 
 class BaseStateObject(TypedDict):
-    timestamp: int
+    timestamp: str
     lastEvent: NotRequired[Optional["ObjectStateWithoutLastEvent"]]
 
 
 class BaseStateDoorbell(TypedDict):
-    timestamp: int
+    timestamp: str
     lastEvent: NotRequired[Optional["DoorbellStateWithoutLastEvent"]]
 
 
 class BaseStateSiren(TypedDict):
-    timestamp: int
+    timestamp: str
     lastEvent: NotRequired[Optional["SirenStateWithoutLastEvent"]]
 
 
 class BaseStateBattery(TypedDict):
-    timestamp: int
+    timestamp: str
     lastEvent: NotRequired[Optional["BatteryStateWithoutLastEvent"]]
 
 
 class BaseStateWithoutLastEvent(TypedDict):
-    timestamp: int
+    timestamp: str
 
 
 class MotionSetEvent(TypedDict):
@@ -539,14 +569,14 @@ class PILRotateOptions(TypedDict):
 
 
 class ImageFormat(TypedDict):
-    format: Literal["rgba", "bgr", "gray"]
+    format: Literal["rgba", "gray"]
 
 
 class ImageCrop(TypedDict):
-    top: float
-    left: float
-    bottom: float
-    right: float
+    top: int
+    left: int
+    width: int
+    height: int
 
 
 class ImageResize(TypedDict):
@@ -595,16 +625,30 @@ class Frame(TypedDict):
 class VideoFrame(Protocol):
     @property
     def metadata(self) -> FrameMetadata: ...
+    @property
     def input_width(self) -> int: ...
+    @property
     def input_height(self) -> int: ...
+    @property
     def input_format(self) -> str: ...
     async def to_buffer(self, options: Optional[ImageOptions] = None) -> bytes: ...
-    async def to_numpy(
-        self, options: Optional[ImageOptions] = None
-    ) -> np.ndarray[Any, Any]: ...
     async def to_image(self, options: Optional[ImageOptions] = None) -> Image.Image: ...
 
 
+SV = TypeVar("SV", bound=StateValue)
+PV = TypeVar("PV", bound=PropertyValue)
+
+
+class CameraStateChangedObject(Generic[SV], Dict[str, SV]):
+    old_state: SV
+    new_state: SV
+
+
+class CameraPropertyObservableObject(Generic[PV], Dict[str, PV]):
+    old_state: PV
+    new_state: PV
+
+
 class CameraDevice(Protocol):
     id: str
     plugin_id: str
@@ -714,8 +758,81 @@ class CameraDevice(Protocol):
         event_data: BatterySetEvent,
     ) -> None: ...
     async def update_state(self, state_name: str, event_data: Any) -> None: ...
-    def on_state_change(self, state_name: StateNames) -> Observable[Any]: ...
-    def on_property_change(self, property_name: str) -> Observable[Any]: ...
+
+    @overload
+    def on_state_change(
+        self, state_name: Literal["light"]
+    ) -> Observable[CameraStateChangedObject[LightState]]: ...
+    @overload
+    def on_state_change(
+        self, state_name: Literal["motion"]
+    ) -> Observable[CameraStateChangedObject[MotionState]]: ...
+    @overload
+    def on_state_change(
+        self, state_name: Literal["audio"]
+    ) -> Observable[CameraStateChangedObject[AudioState]]: ...
+    @overload
+    def on_state_change(
+        self, state_name: Literal["doorbell"]
+    ) -> Observable[CameraStateChangedObject[DoorbellState]]: ...
+    @overload
+    def on_state_change(
+        self, state_name: Literal["siren"]
+    ) -> Observable[CameraStateChangedObject[SirenState]]: ...
+    @overload
+    def on_state_change(
+        self, state_name: Literal["battery"]
+    ) -> Observable[CameraStateChangedObject[BatteryState]]: ...
+    @overload
+    def on_state_change(
+        self, state_name: Literal["object"]
+    ) -> Observable[CameraStateChangedObject[ObjectState]]: ...
+    def on_state_change(  # type: ignore
+        self, state_name: StateNames
+    ) -> Observable[CameraStateChangedObject[StateValue]]: ...
+
+    @overload
+    def on_property_change(
+        self, property_name: Literal["name"]
+    ) -> Observable[CameraPropertyObservableObject[str]]: ...
+    @overload
+    def on_property_change(
+        self,
+        property_name: Literal[
+            "disabled",
+            "isCloud",
+            "hasLight",
+            "hasSiren",
+            "hasBinarySensor",
+            "hasBattery",
+            "hasAudioDetector",
+            "hasMotionDetector",
+            "hasObjectDetector",
+            "hasPrebuffer",
+            "hasIntercom",
+            "hasPtz",
+        ],
+    ) -> Observable[CameraPropertyObservableObject[bool]]: ...
+    @overload
+    def on_property_change(
+        self, property_name: Literal["info"]
+    ) -> Observable[CameraPropertyObservableObject[CameraInformation]]: ...
+    @overload
+    def on_property_change(
+        self, property_name: Literal["type"]
+    ) -> Observable[CameraPropertyObservableObject[CameraType]]: ...
+    @overload
+    def on_property_change(
+        self, property_name: Literal["motionZones", "objectZones"]
+    ) -> Observable[CameraPropertyObservableObject[List[CameraZone]]]: ...
+    @overload
+    def on_property_change(
+        self, property_name: Literal["sources"]
+    ) -> Observable[CameraPropertyObservableObject[List[CameraInput]]]: ...
+    def on_property_change(  # type: ignore
+        self, property_name: CameraPublicProperties
+    ) -> Observable[CameraPropertyObservableObject[PropertyValue]]: ...
+
     def remove_all_listeners(self) -> None: ...
 
 
